using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using SharpCharts.Base.Data;

namespace SharpCharts.Base.Dom
{
    public class DomProfile : DomScriptBase
    {
    	private class Ladder
    	{
            public Ladder(double price)
            {
                this.Price = price;
            }
    		public double Price { get; private set; }
    		public double BidVolume { get; set; }
    		public double AskVolume { get; set; }    		
    	}
    	
        double bidPrice, askPrice;
        double poc = 0.0d;
        double max, min;
        Ladder[] ladders = null;
                
        protected override void Plot(DrawingContext dc, double min, double max, double width, double height)
        {
        	double tickSize = base.TickSize;
        	if (tickSize == 0.0d)
        		return;
        	if (this.poc <= 0)
        		return;
        		
            Ladder[] tmp = this.ladders;
            if (tmp == null)
            	return;

            int len = tmp.Length;

            if (len == 0)
                return;
            
            int highIdx = Math.Max(0, (int)Math.Round((this.max - max) / tickSize, MidpointRounding.AwayFromZero));
            int lowIdx = Math.Min(len, (int)Math.Round((this.max - min) / tickSize, MidpointRounding.AwayFromZero));

            while (highIdx < lowIdx)
            {
                if (highIdx == len)
                    break;
            	Ladder ladder = tmp[highIdx];
            	if (ladder != null)
            	{
                    if (ladder.Price < min)
                        break;
            		double y = CalculateYFromPrice(min, max, ladder.Price, height);
            		if (ladder.BidVolume > 0)
            		{
            			double bidWidth = width / 3 * ladder.BidVolume / this.poc;
            			dc.DrawRectangle(Brushes.LightBlue, null, new Rect(0, y, bidWidth, base.RowHeight));
            		}
            		if (ladder.AskVolume > 0)
            		{
                        double x = width / 3 * ladder.AskVolume / this.poc;
            			dc.DrawRectangle(Brushes.Pink, null, new Rect(width - x, y, x, base.RowHeight));
            		}
                }
                highIdx++;
            }
        }
        
        
 		protected override void OnStateChange()
 		{
 			if (this.State == SharpScript.State.Initialize)
 			{
 				this.ladders = new SharpCharts.Base.Dom.DomProfile.Ladder[0];
 				this.bidPrice = this.askPrice = 0.0d;
 				this.poc = 0L;
 				this.max = double.MinValue;
 				this.min = double.MaxValue;
 			} 
 		}

        
        protected override void OnLevel1(RealTimeDataEventArgs e)
		{
			switch (e.MarketDataType)
			{
				case MarketDataType.Bid:
					this.bidPrice = e.Price;
					break;
				case MarketDataType.Ask:
					this.askPrice = e.Price;
					break;
				case MarketDataType.Last:
					
					if (this.bidPrice == 0.0d || this.askPrice == 0.0d)
						return;
					
					Ladder ladder = GetLadder(e.Price, base.TickSize);
					if (ladder == null)
					{
						return;
					}
					
					if (Math.Abs(e.Price - this.bidPrice) > Math.Abs(askPrice - e.Price)) //traded at bid
					{
						ladder.AskVolume += e.Volume;
						this.poc = Math.Max(this.poc, ladder.AskVolume);
					}
					else if (Math.Abs(e.Price - this.bidPrice) < Math.Abs(askPrice - e.Price))	//traded at ask
					{
						ladder.BidVolume += e.Volume;
						this.poc = Math.Max(this.poc, ladder.BidVolume);
					}
					else
					{
						ladder.BidVolume += e.Volume / 2;
						this.poc = Math.Max(this.poc, ladder.BidVolume);
						
						ladder.AskVolume += e.Volume / 2;
						this.poc = Math.Max(this.poc, ladder.AskVolume);
					}
					
					break;
			}
		}
		
		private Ladder GetLadder(double price, double tickSize)
		{
			if (this.ladders == null)
				return null;
			if (tickSize == 0.0d)
				return null;
			
			if (this.ladders.Length == 0)
			{
				this.min = this.max = price;
				this.ladders = new DomProfile.Ladder[1];
                return this.ladders[0] = new DomProfile.Ladder(price);
			}
			
			if (price > this.max)
			{
				int steps = (int)Math.Round((price - this.max) / tickSize, MidpointRounding.AwayFromZero);
                this.max = price;
                this.ladders = AddUp(this.ladders, steps);
                //if (this.ladders.Length > 0)
                return this.ladders[0] = new DomProfile.Ladder(price);
			}
			else if (price < this.min)
			{
				int steps = (int)Math.Round((this.min - price) / tickSize, MidpointRounding.AwayFromZero);
                this.min = price;
                this.ladders = AddUp(this.ladders, steps * -1);
                //if (this.ladders.Length > 0)
                return this.ladders[this.ladders.Length - 1] = new DomProfile.Ladder(price);
			}
			else
			{
				
				int idx = (int)Math.Round((this.max - price) / tickSize, MidpointRounding.AwayFromZero);
				if (idx >= 0 && idx < this.ladders.Length)
				{
                    if (this.ladders[idx] == null)
                    {
                        this.ladders[idx] = new DomProfile.Ladder(price);
                    }
					return this.ladders[idx];
				}
				
			}
			
			return null;
		}
        
        
        private T[] AddUp<T>(T[] array, int steps)
         {
             T[] tmp = new T[array.Length + Math.Abs(steps)];
 
             if (steps >= 0)  //price increase
             {
                 array.CopyTo(tmp, steps);
                 
             }
             else
             {
                 array.CopyTo(tmp, 0);
             }
             return tmp;
         }
         
    }
}